@@ -111,10 +111,22 @@
 ##' @param ncpus integer: number of processes to be used in parallel operation.
 ##' @param cl An optional \code{parallel} or \code{snow} cluster for use if \code{parallel = "snow"}. If not supplied, a cluster on the local machine is created for the duration of the \code{Score} call.
 ##' @param progress.bar Style for \code{txtProgressBar}. Can be 1,2,3 see \code{help(txtProgressBar)} or NULL to avoid the progress bar.
+##' @param errorhandling Argument passed as \code{.errorhandling} to foreach. Default is \code{"pass"}.
 ##' @param keep list of characters (not case sensitive) which determines additional output.
 ##' \code{"residuals"} provides Brier score residuals and
 ##' \code{"splitindex"} provides sampling index used to split the data into training and validation sets. It is a function, whose argument is the bootstrap sample, which one wishes to look at.
-##' \code{"vcov"} provides the variance-covariance matrix of the estimated parameters.
+##' \code{"vcov"} provides the variance-covariance matrix for the estimates.
+##' \code{"iid"} provides the estimated influence function of the estimates.
+##' @param breaks Break points for computing the ROC curve when crossvalidation is applied. Defaults to
+##' \code{seq(0,1,.01)}, although it is not implmeneted for \code{split.method = "loob"} or \code{split.method = "cvk"} and \code{B} > 1 yet.
+##' @param roc.method Method for averaging ROC curves across data splits.
+##' If \code{'horizontal'} average crossvalidated specificities for fixed sensitivity values,
+##' specified in \code{roc.grid}, otherwise, if \code{'vertical'},
+##' average crossvalidated sensitivities for fixed specificity values.
+##' See Fawcett, T. (2006) for details.
+##' @param roc.grid Grid points for the averaging of ROC curves.
+##' A sequence of values at which to compute averages across the ROC curves
+##' obtained for different data splits during crossvalidation. 
 ##' @param censoring.save.memory Only relevant in censored data where censoring weigths are obtained with
 ##' Cox regression and argument \code{conservative} is set to \code{FALSE}. If \code{TRUE}, save memory by not storing the influence function
 ##' of the cumulative hazard of the censoring as a matrix when calculating standard errors
@@ -128,12 +140,29 @@
 ##'  specify additional arguments for the function riskRegression::predictRisk.rfsrc which will pass
 ##'  these on to the function randomForestSRC::predict.rfsrc. A specific example in this case would be
 ##'  \code{list(rfsrc=list(na.action="na.impute"))}.
-##'  
-##'
+##' 
 ##'  A more flexible approach is to write a new predictRisk S3-method. See Details.
-##' @param errorhandling Argument passed as \code{.errorhandling} to foreach. Default is \code{"pass"}.
 ##' @param debug Logical. If \code{TRUE} indicate landmarks in progress of the program.
-##' @param ... Named list containging additional arguments that are passed on to the \code{predictRisk} methods corresponding to object. See examples.
+##' @param censoring.save.memory Only relevant in censored data where censoring weigths are obtained with
+##' Cox regression and argument \code{conservative} is set to \code{FALSE}. If \code{TRUE}, save memory by not storing the influence function
+##' of the cumulative hazard of the censoring as a matrix when calculating standard errors
+##' with Cox censoring. This can allow one to use \code{Score} on larger test data sets,
+##' but may be slower.
+##' @param breaks Break points for computing the Roc curve. Defaults to
+#' \code{seq(0,1,.01)} when some form of crossvalidation is applied, otherwise
+#' to all unique values of the predictive marker.
+##' @param roc.method Method for averaging ROC curves across data splits.
+#' If \code{'horizontal'} average crossvalidated specificities for fixed sensitivity values,
+#' specified in \code{roc.grid}, otherwise, if  \code{'vertical'},
+#' average crossvalidated specificities for fixed sensitivity values.
+#' See Fawcett, T. (2006) for details.
+##' @param roc.grid Grid points for the averaging of ROC curves.
+#' A sequence of values at which to compute averages across the ROC curves
+#' obtained for different data splits during crossvalidation.
+##' @param cutpoints If not \code{NULL}, estimates and standard errors of the TPR (True Positive Rate), 
+##' FPR (False Positive Rate), PPV (Positive Predictive Value), and NPV (Negative Predictive Value) 
+##' are given at the \code{cutpoints}. These values are saved in \code{object$AUC$res.cut}. 
+##' @param ... Named list containing additional arguments that are passed on to the \code{predictRisk} methods corresponding to object. See examples.
 ##' @return List with scores and assessments of contrasts, i.e.,
 ##'     tests and confidence limits for performance and difference in performance (AUC and Brier),
 ##'     summaries and plots. Most elements are in\code{data.table} format.
@@ -435,6 +464,9 @@
 #' Michael W Kattan and Thomas A Gerds. The index of prediction accuracy: an
 #' intuitive measure useful for evaluating risk prediction models. Diagnostic
 #' and Prognostic Research, 2(1):7, 2018.
+#' 
+#' Fawcett, T. (2006). An introduction to ROC analysis. Pattern
+#' Recognition Letters, 27, 861-874.
 ##'
 ##' @export Score.list
 ##' @export
@@ -484,6 +516,10 @@ Score.list <- function(object,
                        predictRisk.args,
                        debug=0L,
                        censoring.save.memory = FALSE,
+                       breaks = seq(0,1,.01), 
+                       roc.method='vertical',
+                       roc.grid=switch(roc.method,"vertical"=seq(0,1,.01),"horizontal"=seq(1,0,-.01)),
+                       cutpoints = NULL,
                        ...){
     se.conservative=IPCW=IF.AUC.conservative=IF.AUC0=IF.AUC=IC0=Brier=AUC=casecontrol=se=nth.times=time=status=ID=WTi=risk=IF.Brier=lower=upper=crossval=b=time=status=model=reference=p=model=pseudovalue=ReSpOnSe=residuals=event=j=NULL
 
@@ -524,9 +560,13 @@ Score.list <- function(object,
     }
     ## Plots <- lapply(plots,grep,c("Roc","Cal"),ignore.case=TRUE,value=TRUE)
     if ("ROC" %in% plots) {
+        ROC <- TRUE
         ## add AUC if needed
         if (!("AUC" %in% metrics)) metrics <- c(metrics,"AUC")
     }
+    else {
+      ROC <- FALSE
+    }
     if ("Calibration" %in% plots) {
         ## add pseudo if needed
         if (!("pseudo" %in% cens.method)) cens.method <- c(cens.method,"pseudo")
@@ -564,9 +604,10 @@ Score.list <- function(object,
     }
     if (missing(data)){stop("Argument data is missing.")}
     if (data.table::is.data.table(data))
-        data <- copy(data)
-    else
-        data <- data.table::setDT(data)
+        data <- data.table::copy(data)
+    else{
+        data <- data.table::as.data.table(data)
+    }
     responseFormula <- stats::update(formula,~1)
     ## if (missing(event)) event <- 1
     responsevars <- all.vars(responseFormula)
@@ -687,9 +728,6 @@ c.f., Chapter 7, Section 5 in Gerds & Kattan 2021. Medical risk prediction model
                 conservative=TRUE
             }
         }
-        if ("ROC" %in% plots){
-            warning("Cannot (not yet) do ROC analysis in combination with internal validation\n. Check devtools::install_github('tagteam/riskRegression') for progress.")
-        }
     }
 
     # }}}
@@ -760,9 +798,11 @@ c.f., Chapter 7, Section 5 in Gerds & Kattan 2021. Medical risk prediction model
         if("vcov" %in% tolower(keep)) keep.vcov=TRUE else keep.vcov = FALSE
         if ("splitindex" %in% tolower(keep)) keep.splitindex=TRUE else keep.splitindex = FALSE
         if ("cv" %in% tolower(keep)) keep.cv=TRUE else keep.cv = FALSE
+        if ("iid" %in% tolower(keep)) keep.iid=TRUE else keep.iid = FALSE
     }else{
         keep.residuals=FALSE
         keep.vcov=FALSE
+        keep.iid=FALSE
         keep.cv=FALSE
         keep.splitindex=FALSE
     }
@@ -962,6 +1002,7 @@ c.f., Chapter 7, Section 5 in Gerds & Kattan 2021. Medical risk prediction model
                                       multi.split.test=multi.split.test,
                                       keep.residuals=keep.residuals,
                                       keep.vcov=keep.vcov,
+                                      keep.iid=keep.iid,
                                       dolist=dolist,
                                       probs=probs,
                                       metrics=metrics,
@@ -969,9 +1010,11 @@ c.f., Chapter 7, Section 5 in Gerds & Kattan 2021. Medical risk prediction model
                                       summary=summary,
                                       ibs=ibs,
                                       ipa = ipa,
-                                      ROC=FALSE,
+                                      ROC=ROC,
                                       MC=Weights$IC,
-                                      IC.data=Weights$IC.data)
+                                      IC.data=Weights$IC.data,
+                                      breaks=NULL,
+                                      cutpoints=cutpoints)
         if (debug) message("computed apparent performance")
     }
     # }}}
@@ -1154,6 +1197,7 @@ if (split.method$internal.name%in%c("BootCv","LeaveOneOutBoot","crossval")){
                                        multi.split.test=multi.split.test,
                                        keep.residuals=FALSE,
                                        keep.vcov=FALSE,
+                                       keep.iid = FALSE,
                                        dolist=dolist,
                                        probs=probs,
                                        metrics=metrics,
@@ -1161,9 +1205,10 @@ if (split.method$internal.name%in%c("BootCv","LeaveOneOutBoot","crossval")){
                                        summary=summary,
                                        ibs=ibs,
                                        ipa=ipa,
-                                       ROC=FALSE,
+                                       ROC=ROC,
                                        MC=Weights$IC,
-                                       IC.data=Weights$IC.data)
+                                       IC.data=Weights$IC.data,
+                                       breaks=NULL)
     }
     else if (split.method$name=="LeaveOneOutBoot" | split.method$internal.name =="crossval"){  ## Testing if the crossval works in this loop
         message(paste0("Calculating the performance metrics in long format\nlevel-1 data with ",
@@ -1198,15 +1243,110 @@ if (split.method$internal.name%in%c("BootCv","LeaveOneOutBoot","crossval")){
                                                                     ID,
                                                                     cause)})
       names(crossvalPerf) <- metrics
+      ## copy paste from bootcv; same method to calculate ROC
+      if (ROC){
+        ## implement ROC here
+        if (parallel=="snow") exports <- c("DT.B","N.b","cens.model","multi.split.test") else exports <- NULL
+        if (!is.null(progress.bar)){
+          if (!(progress.bar %in% c(1,2,3))) progress.bar <- 3
+          pb1 <- txtProgressBar(max = B, style = progress.bar,width=20)
+          message(paste0("Calculating the ROC curve in ",B," validation data sets"))
+        }
+        crossval <- foreach::foreach(j=1:B,.export=exports,.packages="data.table",.errorhandling=errorhandling) %dopar%{
+          DT.b <- DT.B[b==j]
+          N.b <- length(unique(DT.b[["ID"]]))
+          if(!is.null(progress.bar)){
+            setTxtProgressBar(pb1, j)
+          }
+          computePerformance(DT=DT.b,
+                             N=N.b,
+                             NT=NT,
+                             NF=NF,
+                             models=list(levels=mlevs,labels=mlabels),
+                             response.type=response.type,
+                             times=times,
+                             jack=jack,
+                             cens.type=cens.type,
+                             cause=cause,
+                             states=states,
+                             alpha=alpha,
+                             se.fit=FALSE,
+                             conservative=TRUE,
+                             cens.model=cens.model,
+                             multi.split.test=multi.split.test,
+                             keep.residuals=FALSE,
+                             keep.vcov=FALSE,
+                             keep.iid =FALSE,
+                             dolist=dolist,
+                             probs=probs,
+                             metrics="AUC",
+                             plots=plots,
+                             summary=summary,
+                             ibs=ibs,
+                             ipa=ipa,
+                             ROC=ROC,
+                             MC=Weights$IC,
+                             IC.data=Weights$IC.data, 
+                             breaks=breaks)
+        }
+        cumROC <- lapply(crossval,function(x) x[["ROC"]][["plotframe"]])
+        TPR=FPR=NULL
+        fancy.fun <- function(risk,TPR,FPR,roc.method,roc.grid){
+          if (roc.method == "vertical"){
+            temp <- stats::approx(x=FPR,
+                                  y=TPR,
+                                  xout=roc.grid,
+                                  ties=median,
+                                  yleft=0,
+                                  yright=1)$y
+            list(risk = rep(NA,length(roc.grid)),TPR=temp, FPR=roc.grid)
+          }
+          else {
+            temp <- stats::approx(x=TPR,
+                                  y=FPR,
+                                  xout=roc.grid,
+                                  ties=median,
+                                  yleft=0, #have to consider if yleft and yright have to be modified.
+                                  yright=1)$y
+            data.table(risk = rep(NA,length(roc.grid)),TPR=roc.grid, FPR=temp)
+          }
+        }
+        if (response.type == "binary"){
+          cumROC <- lapply(cumROC,function(x) x[,fancy.fun(risk,TPR,FPR,roc.method=roc.method,roc.grid=roc.grid),by=list(model)])
+          if (roc.method == "vertical"){
+            cumROC <- rbindlist(cumROC)[,lapply(.SD, mean),by=list(model,FPR)]
+          }
+          else {
+            cumROC <- rbindlist(cumROC)[,lapply(.SD, mean),by=list(model,TPR)]
+          }
+        }
+        else {
+          cumROC <- lapply(cumROC,function(x) x[,fancy.fun(risk,TPR,FPR,roc.method=roc.method,roc.grid=roc.grid),by=list(model,times)])
+          if (roc.method == "vertical"){
+            cumROC <- rbindlist(cumROC)[,lapply(.SD, mean),by=list(model,times,FPR)]
+          }
+          else {
+            cumROC <- rbindlist(cumROC)[,lapply(.SD, mean),by=list(model,times,TPR)]
+          }
+        }
+        setorder(cumROC, cols = "TPR")
+        ROC.res <- list(plotframe=cumROC,plotmethod="ROC")
+        class(ROC.res) <- "scoreROC"
+        crossvalPerf[["ROC"]] <- ROC.res
+      }
     }
-    if (split.method$name=="BootCv"){
+    else if (split.method$name=="BootCv"){
         # {{{ bootcv
+        ## implement ROC here
         if (parallel=="snow") exports <- c("DT.B","N.b","cens.model","multi.split.test") else exports <- NULL
         if (!is.null(progress.bar)){
             if (!(progress.bar %in% c(1,2,3))) progress.bar <- 3
             pb1 <- txtProgressBar(max = B, style = progress.bar,width=20)
             message(paste0("Calculating the performance metrics in ",B," validation data sets"))
         }
+        if (!("ROC" %in% plots)){
+          breaks <- NULL
+        }
         crossval <- foreach::foreach(j=1:B,.export=exports,.packages="data.table",.errorhandling=errorhandling) %dopar%{
             DT.b <- DT.B[b==j]
             N.b <- length(unique(DT.b[["ID"]]))
@@ -1231,6 +1371,7 @@ if (split.method$internal.name%in%c("BootCv","LeaveOneOutBoot","crossval")){
                                multi.split.test=multi.split.test,
                                keep.residuals=FALSE,
                                keep.vcov=FALSE,
+                               keep.iid = FALSE,
                                dolist=dolist,
                                probs=probs,
                                metrics=metrics,
@@ -1238,15 +1379,63 @@ if (split.method$internal.name%in%c("BootCv","LeaveOneOutBoot","crossval")){
                                summary=summary,
                                ibs=ibs,
                                ipa=ipa,
-                               ROC=FALSE,
+                               ROC=ROC,
                                MC=Weights$IC,
-                               IC.data=Weights$IC.data)
+                               IC.data=Weights$IC.data, 
+                               breaks=breaks)
         }
         if (!is.null(progress.bar)){
             cat("\n")
         }
         crossvalPerf <- lapply(metrics,function(m) crossvalPerf.bootcv(m,crossval,se.fit,multi.split.test,keep.cv,byvars,alpha))
         names(crossvalPerf) <- metrics
+        
+        if ("ROC" %in% plots){
+            cumROC <- lapply(crossval,function(x) x[["ROC"]][["plotframe"]])
+            TPR=FPR=NULL
+            fancy.fun <- function(risk,TPR,FPR,roc.method,roc.grid){
+                if (roc.method == "vertical"){
+                    temp <- stats::approx(x=FPR,
+                                          y=TPR,
+                                          xout=roc.grid,
+                                          ties=median,
+                                          yleft=0,
+                                          yright=1)$y
+                    list(risk = rep(NA,length(roc.grid)),TPR=temp, FPR=roc.grid)
+                }
+                else {
+                    temp <- stats::approx(x=TPR,
+                                          y=FPR,
+                                          xout=roc.grid,
+                                          ties=median,
+                                          yleft=0, #have to consider if yleft and yright have to be modified.
+                                          yright=1)$y
+                    data.table(risk = rep(NA,length(roc.grid)),TPR=roc.grid, FPR=temp)
+                }
+            }
+            if (response.type == "binary"){
+                cumROC <- lapply(cumROC,function(x) x[,fancy.fun(risk,TPR,FPR,roc.method=roc.method,roc.grid=roc.grid),by=list(model)])
+                if (roc.method == "vertical"){
+                    cumROC <- rbindlist(cumROC)[,lapply(.SD, mean),by=list(model,FPR)]
+                }
+                else {
+                    cumROC <- rbindlist(cumROC)[,lapply(.SD, mean),by=list(model,TPR)]
+                }
+            }
+            else {
+                cumROC <- lapply(cumROC,function(x) x[,fancy.fun(risk,TPR,FPR,roc.method=roc.method,roc.grid=roc.grid),by=list(model,times)])
+                if (roc.method == "vertical"){
+                    cumROC <- rbindlist(cumROC)[,lapply(.SD, mean),by=list(model,times,FPR)]
+                }
+                else {
+                    cumROC <- rbindlist(cumROC)[,lapply(.SD, mean),by=list(model,times,TPR)]
+                }
+            }
+            setorder(cumROC, cols = "TPR")
+            ROC.res <- list(plotframe=cumROC,plotmethod="ROC")
+            class(ROC.res) <- "scoreROC"
+            crossvalPerf[["ROC"]] <- ROC.res
+        }
         if (ipa==TRUE){
             if (response.type=="binary")
                 crossvalPerf[["Brier"]][["score"]][,IPA:=1-Brier/Brier[model=="Null model"]]
@@ -1287,17 +1476,24 @@ if (split.method$internal.name%in%c("BootCv","LeaveOneOutBoot","crossval")){
     }
     # }}}
 }
-
     # }}}
     # {{{ the output object
     if (split.method$internal.name=="noplan"){
         if (keep.residuals==TRUE){
-            noSplit$Brier$residuals[,model:=factor(model,levels=mlevs,mlabels)]
+            if("Brier" %in% metrics)
+                noSplit$Brier$residuals[,model:=factor(model,levels=mlevs,mlabels)]
+        }
+        if (keep.iid==TRUE){
+            if("Brier" %in% metrics)
+                noSplit$Brier$iid.decomp[,model:=factor(model,levels=mlevs,mlabels)]
+            if("AUC" %in% metrics)
+                noSplit$AUC$iid.decomp[,model:=factor(model,levels=mlevs,mlabels)]
         }
         output <- noSplit
     } else{
         output <- crossvalPerf
     }
+
     models <- mlevs
     names(models) <- mlabels
     if (keep.vcov){
