@@ -1,189 +1,159 @@
 # Function to calculate cross-validation performance
 
-crossvalPerf.loob.AUC <- function(times,mlevs,se.fit,response.type,NT,Response,cens.type,Weights,split.method,N,B,DT.B,data,dolist,alpha,byvars,mlabels,conservative,cens.model) {
+crossvalPerf.loob.AUC <- function(times,mlevs,se.fit,response.type,NT,Response,cens.type,Weights,split.method,N,B,DT.B,data,dolist,alpha,byvars,mlabels,conservative,cens.model,cause) {
   # initializing output
-  AUC <- ReSpOnSe <- status <- ID <- model <- b <- risk <- casecontrol <- IF.AUC <- IF.AUC0 <- se <- IF.AUC.conservative <- se.conservative <- lower <- upper <- NF <- reference  <-  event <- status0 <- NULL
-  if (response.type=="binary")
+  bfold <- fold <- AUC <- ReSpOnSe <- status <- ID <- model <- b <- risk <- casecontrol <- IF.AUC <- IF.AUC0 <- se <- IF.AUC.conservative <- se.conservative <- lower <- upper <- NF <- reference  <-  event <- status0 <- NULL
+  if (response.type=="binary") {
     auc.loob <- data.table(expand.grid(times=0,model=mlevs)) #add times to auc.loob; now we can write less code for the same thing!
-  else
-    auc.loob <- data.table(expand.grid(times=times,model=mlevs))
+    times <- 0
+  }
+  auc.loob <- data.table::data.table(expand.grid(times=times,model=mlevs))
   auc.loob[,AUC:=as.numeric(NA)]
   ## for each pair of individuals sum the concordance of the bootstraps where *both* individuals are out-of-bag
   ## divide by number of times the pair is out-of-bag later
   aucDT <- NULL
-  ## preparation of outcome status at time horizon(s)
-  if (response.type=="binary"){
-    NT <- 1
-    cens.model <- "none"
+  
+  redoSplitIndex <- function(x, N){
+    res <- rep(TRUE, N)
+    res[unique(x)] <- FALSE
+    res
   }
+  ind.mat <- do.call("cbind",(lapply(1:B,split.method$index)))
+  if (split.method$internal.name == "LeaveOneOutBoot"){
+    split.index <- apply(ind.mat,2,function(x) redoSplitIndex(x,N))
+  }
+  else {
+    setorder(DT.B, b, ID)
+    DT.B$fold <- rep(c(ind.mat), each=NT*length(mlevs))
+    DT.B[, bfold:=as.numeric(interaction(b,fold))] # construct new function
+    setorder(DT.B, bfold, ID)
+    k <- split.method$k
+    split.index <- do.call("cbind",lapply(1:k, function(i) ind.mat==i))
+  }
+  cause <- as.numeric(cause)
+  rm(ind.mat)
   for (s in 1:NT){
     if (response.type=="binary"){
       t <- 0
       ## the following indices have to be logical!!!
       cases.index <- Response[,ReSpOnSe==1]
       controls.index <- !cases.index
-      cc.status <- factor(cases.index,levels=c(TRUE,FALSE),labels=c("case","control"))
     }else{
       t <- times[s]
-      if (response.type=="survival"){
-        ## event of interest before times
-        ## the following indices have to be logical!!!
-        cases.index <- Response[,time<=t & status==1]
-        controls.index = controls.index1 = Response[,time>t]
-        controls.index2 <- rep(FALSE, N)
-        cc.status <- factor(rep("censored",N),levels=c("censored","case","control"))
-        cc.status[cases.index] <- "case"
-        cc.status[controls.index] <- "control"
+      if (response.type == "survival"){
+        Response$status0 <- Response$status
       }
-      else{ ## competing risks
-        ## event of interest before times
-        ## the following indices have to be logical!!!
-        # Response[,status0:=status*event]
-        cases.index <- Response[,time<=t & event==1]
-        controls.index1 <- Response[,time>t]
-        controls.index2 <-  Response[,event==2 & time <= t]
-        controls.index <- controls.index1 | controls.index2
-        cc.status <- factor(rep("censored",N),levels=c("censored","case","control"))
-        cc.status[cases.index] <- "case"
-        cc.status[controls.index] <- "control"
+      else { # competing risks
+        oldEvent <- Response$event
+        causeID <- oldEvent == cause
+        cause1ID <- oldEvent == 1
+        oldEvent[causeID] <- 1
+        oldEvent[cause1ID] <- cause
+        Response$status0 <- Response$status * oldEvent
+        # need a status variable with value
+        # 0 = censored
+        # 1 = cause of interest
+        # 2 = other causes
       }
+      cases.index <- Response[,time<=t & status0==1]
+      controls.index1 <- Response[,time>t]
+      controls.index2 <-  Response[,status0==2 & time <= t]
+      controls.index <- controls.index1 | controls.index2
     }
     # censoring weights
-    if (cens.type=="rightCensored"){ #this maybe does not work with competing risks
-      ## IPCW
-      weights.cases <- cases.index/Weights$IPCW.subject.times
-      if (response.type == "survival"){
-        if (Weights$method=="marginal"){
-          weights.controls <- controls.index/Weights$IPCW.times[s]
-        }else{
-          weights.controls <- controls.index/Weights$IPCW.times[,s]
-        }
+    if (cens.type=="rightCensored"){ 
+      if (Weights$method!="cox"){
+        Wt <- Weights$IPCW.times[s]
+      }else{
+        Wt <- Weights$IPCW.times[,s]
       }
-      else {
-        if (Weights$method=="marginal"){
-          weights.controls <- 1/Weights$IPCW.times[s] * controls.index1  + 1/Weights$IPCW.subject.times * controls.index2
-        }else{
-          weights.controls <- controls.index/Weights$IPCW.times[,s]*controls.index1 +  1/Weights$IPCW.subject.times * controls.index2
-        }
-      }
-      weightMatrix <- outer(weights.cases[cases.index], weights.controls[controls.index], "*")
+      weights <- as.numeric(1/Wt * controls.index1  + 1/Weights$IPCW.subject.times * (cases.index | controls.index2))
     }else{ ## uncensored
-      weights.cases <- cases.index/1
-      weights.controls <- controls.index/1
-      weightMatrix <- outer(weights.cases[cases.index], weights.controls[controls.index], "*")
+      weights <- rep(1,N)
     }
-    w.cases <- weights.cases[cases.index]
-    w.controls <- weights.controls[controls.index]
-    Phi <- (1/N^2)*sum(w.cases)*sum(w.controls)
+    w.cases <- weights[cases.index]
+    w.controls <- weights[controls.index]
+    n.cases <- sum(cases.index)
+    n.controls <- sum(controls.index)
+    ID.case <- data[cases.index,]$ID
+    ID.controls <- data[controls.index,]$ID
+    muCase <- sum(w.cases)
+    muControls <- sum(w.controls)
+    Phi <- (1/N^2)*muCase*muControls
     for (mod in mlevs){
       if (mod == 0){
         aucLPO <- 0.5
         auc.loob[times==t&model==mod,AUC:=aucLPO]
+        DT.B <- DT.B[model != 0]
       }
       else {
-        if (split.method$internal.name=="crossval"){
-          stop("Cannot yet calculate AUC in this case. Use split.method 'loob' or 'bootcv' instead.")
+        if (response.type == "binary"){ # no reason for the below trick with k fold, as level one data set is approximately N*B
+          DTrisk <- DT.B[model == mod][["risk"]]
         }
-        Ib <- matrix(0, sum(cases.index), sum(controls.index))
-        auc <- matrix(0, sum(cases.index), sum(controls.index))
-        for (u in 1:B){## cannot use b as running index because b==b does not work in data.table
-          ## test <- DT.B[model==mod&times==t&b==u]
-          # when B is too low it may happen that some subjects are never oob
-          oob <- match(1:N,unique(split.method$index[,u]),nomatch=0)==0
-          ## to use the cpp function AUCijFun we
-          ## need a vector of length equal to the number of cases (cases.index) for the current time point
-          ## which has arbitrary values in places where subjects are inbag and the predicted risks
-          ## for those out-of-bag. need another vector for controls.
-          riskset <- data.table::data.table(ID=1:N,casecontrol=cc.status,oob=oob)
-          data.table::setkey(riskset,ID)
-          if (response.type=="binary"){
-            oob.risk <- DT.B[model==mod&b==u,data.table::data.table(ID,risk)]
-          }else{
-            oob.risk <- DT.B[model==mod&times==t&b==u,data.table::data.table(ID,risk)]
-          }
-          data.table::setkey(oob.risk,ID)
-          riskset <- oob.risk[riskset]
-          riskset[is.na(risk),risk:=-1]
-          Ib.ij <- outer((cases.index*oob)[cases.index],(controls.index*oob)[controls.index],"*")
-          auc.ij <- AUCijFun(riskset[casecontrol=="case",risk],riskset[casecontrol=="control",risk])*Ib.ij #case > control
-          ## Ib.ij is 1 when the pair out of bag
-          ## print(head(oob))
-          ## print(auc.ij[1:5,1:5])
-          auc <- auc+auc.ij
-          Ib <- Ib + Ib.ij
+        else {
+          DTrisk <- DT.B[model == mod & times == t][["risk"]]
         }
-        auc <- (auc*weightMatrix)/Ib
-        # FIXME: why are there NA's?
-        auc[is.na(auc)] <- 0
-        nu1tauPm <- (1/N^2)*sum(auc)
+        if (split.method$internal.name == "crossval"){
+          n.col <- k*B
+        } else {
+          n.col <- B
+        }
+        risk.mat <- matrix(NA,nrow=N,ncol=n.col)
+        risk.mat[split.index] <- DTrisk
+        res <- aucLoobFun(ID.case,ID.controls,risk.mat,split.index,weights)
+        ic0Case <- res[["ic0Case"]]
+        ic0Control <- res[["ic0Control"]]
+        warn <- res[["warn"]]
+        nu1tauPm <- (1/N^2)*sum(ic0Case)
         ## Leave-one-pair-out bootstrap estimate of AUC
         aucLPO <- nu1tauPm*(1/Phi)
         # following section should be cleaned up(!!!)
         auc.loob[times==t&model==mod,AUC:=aucLPO]
       }
       if (se.fit==1L){ ## should clean this up when cv has been fixed !
-        if (!(cens.model %in% c("none", "KaplanMeier","cox")) && response.type != "binary" && !conservative[[1]]) stop("Censoring model not supported when conservative = TRUE")
-        id.cases <- data[["ID"]][cc.status=="case"]
-        id.controls <- data[["ID"]][cc.status=="control"]
-        id.censored <- data[["ID"]][cc.status=="censored"]
         if (mod == 0){
-          ic <- rep(0,N)
-          this.aucDT <- data.table(model=mod,times=t,ID = c(id.cases,id.controls,id.censored))
+          aucDT <- data.table::data.table(model=mod,times=t,IF.AUC=rep(0,N))
         }
-        else if (cens.model == "KaplanMeier"){
-          if (response.type == "survival"){
-            data[,status0:=status]
+        else {
+          IF.AUC0 <- rep(0,N)
+          IF.AUC0[cases.index] <- 1/(Phi*N)*ic0Case
+          IF.AUC0[controls.index] <- 1/(Phi*N)*ic0Control
+          if (!response.type == "binary" && !cens.type == "uncensored" && !conservative[[1]]){
+            IFcalculationList <- list(ic0Case = ic0Case,
+                                      ic0Control = ic0Control,
+                                      weights = weights, 
+                                      muCase = muCase,
+                                      muControls = muControls, 
+                                      nu = nu1tauPm,
+                                      firsthit = sindex(data[["time"]],t)-1,
+                                      cases = cases.index,
+                                      controls =controls.index,
+                                      controls1 = controls.index1,
+                                      controls2 = controls.index2)
+            icPart <- getInfluenceFunction.AUC.censoring.term(time = Response[["time"]],
+                                                              event = Response[["status0"]],
+                                                              t= t,
+                                                              IFcalculationList = IFcalculationList,
+                                                              MC = Weights$IC, 
+                                                              cens.model = cens.model, 
+                                                              Wt = Weights$IPCW.times[s], 
+                                                              auc = aucLPO, 
+                                                              nth.times = s)
           }
           else {
-            data[,status0:=status*event]
+            icPart <- 0
           }
-          this.aucDT <- data.table(model=mod,times=t,ID = c(id.cases,id.controls,id.censored))
-          ic <- getInfluenceFunctionAUCKMCensoringCVPart(data[["time"]],data[["status0"]],t,Weights$IPCW.subject.times, Weights$IPCW.times[s], auc,nu1tauPm)
-        }
-        else {
-          ic0Case <- rowSums(auc)
-          ic0Control <- colSums(auc)
-          ic0 <- (1/(Phi*N))*c(ic0Case, ic0Control)
-          this.aucDT <- data.table(model=mod,times=t,ID = c(id.cases,id.controls,id.censored), IF.AUC0 = c(ic0, rep(0,length(id.censored))))
-        }
-        aucDT <- rbindlist(list(aucDT,this.aucDT),use.names=TRUE,fill=TRUE)
-        if (cens.model == "cox" && !conservative[[1]] && mod != 0){ 
-          if (Weights$IC$saveCoxMemory){
-            wdata <- Weights$IC[[3]]
-            fit <- Weights$IC[[2]]
-            TiMinus <- Weights$IC[[4]]
-            ic0CaseOld <- rep(0,N)
-            ic0ControlOld <- rep(0,N)
-            ic0CaseOld[cases.index] <- 1/w.cases * ic0Case#ic0CaseOld[cases.index]*w.cases
-            ic0ControlOld[controls.index] <- 1/w.controls * ic0Control
-            Wbeforet <- (1/(Phi*N^2))*(ic0CaseOld*weights.cases*cases.index+ic0ControlOld*weights.controls*controls.index2)-
-              (1/N)*(aucLPO/Phi)*((cases.index)*weights.cases*(1/N)*sum(w.controls)+ (controls.index2)*weights.controls*(1/N)*sum(w.cases))
-            Waftert <- (1/(Phi*N^2))*ic0ControlOld*weights.controls*controls.index1- 
-              (1/N)*(aucLPO/Phi)*(controls.index1)*weights.controls*(1/N)*sum(w.cases)
-            icPart <- predictCoxWeights(fit, diag=TRUE,newdata = wdata, times = TiMinus,weights=Wbeforet, isBeforeTau = TRUE, tau = t)+
-              predictCoxWeights(fit, diag=FALSE,newdata = wdata,times = t,weights=Waftert)
-          }
-          else {
-            ic.weights <- Weights$IC[[2]][[s]]
-            icPart <- as.numeric(rowSumsCrossprod(as.matrix(1/(Phi*N^2)*ic0Case-(aucLPO/Phi)*(1/N^2)*sum(w.controls)*w.cases), ic.weights[cases.index,], 0)) +
-              as.numeric(rowSumsCrossprod(as.matrix(1/(Phi*N^2)*ic0Control-(aucLPO/Phi)*(1/N^2)*sum(w.cases)*w.controls),ic.weights[controls.index,],0))
-          }
-        }
-        else {
-          icPart <- 0
-        }
-        icPhi1 <- (aucLPO/Phi)*((weights.cases)*(1/N)*sum(weights.controls)+(weights.controls)*(1/N)*sum(weights.cases))
-        data.table::setkey(aucDT,model,times,ID)
-        if (cens.model == "KaplanMeier" || mod == 0){
-          aucDT[model==mod&times==t, IF.AUC:=ic]
-        }
-        else {
-          aucDT[model==mod&times==t, IF.AUC:=IF.AUC0+icPart-icPhi1]
+          icPhi1 <- weights*(aucLPO/Phi)*(cases.index*(1/N)*muControls+controls.index*(1/N)*muCase)
+          this.aucDT <- data.table::data.table(model=mod,times=t,IF.AUC=IF.AUC0+icPart-icPhi1)
+          aucDT <- rbindlist(list(aucDT,this.aucDT),use.names=TRUE,fill=TRUE)
         }
         auc.loob[model==mod&times==t,se:= sd(aucDT[model==mod&times==t,IF.AUC])/sqrt(N)]
       }
     }
   }
+  if (warn){
+    warning("Some pairs of subjects are never out of bag at the same time. \n You should increase the number of bootstrap replications (argument 'B') ")
+  }
   if (response.type == "binary"){
     # remove times again
     auc.loob[,times:=NULL] 
@@ -272,7 +242,9 @@ crossvalPerf.loob.Brier <- function(times,mlevs,se.fit,response.type,NT,Response
   DT.B <- DT.B[,data.table::data.table(risk=mean(risk),residuals=sum(residuals)),by=c(byvars,"ID")]
   ## get denominator
   if (split.method$name=="LeaveOneOutBoot"){
-    Ib <- split.method$B-tabulate(unlist(apply(split.method$index,2,unique)))
+    ind.mat <- do.call("cbind",(lapply(1:B,split.method$index)))
+    Ib <- split.method$B-tabulate(unlist(apply(ind.mat,2,unique)))
+    rm(ind.mat)
     ## REMOVE ME
     ## Ib <- Ib[order(data$ID)]
     if (any(Ib==0)) {
@@ -299,7 +271,7 @@ crossvalPerf.loob.Brier <- function(times,mlevs,se.fit,response.type,NT,Response
     DT.B[,IC0:=residuals-Brier]
     ## se.brier <- DT.B[,list(se=sd(IC0, na.rm=TRUE)/sqrt(N)),by=byvars]
     ## DT.B[,Brier:=NULL]
-    if (cens.type[1]=="rightCensored"){
+    if (cens.type[1]=="rightCensored" && !conservative){
       ## this is a new DT.B
       DT.B <- cbind(data[,1:response.dim,with=FALSE],DT.B)
       DT.B[,nth.times:=as.numeric(factor(times))]
@@ -333,16 +305,14 @@ crossvalPerf.loob.Brier <- function(times,mlevs,se.fit,response.type,NT,Response
           DT.B[,status0:=status*event]
         }
         DT.B[,IF.Brier:=getInfluenceCurve.Brier(t=times[1],
-                                              time=time,
-                                              IC0,
-                                              residuals=residuals,
-                                              WTi=WTi,
-                                              Wt=Wt,
-                                              IC.G=Weights$IC,
-                                              cens.model=cens.model,
-                                              conservative = conservative,
-                                              nth.times=nth.times[1],
-                                              event = status0),by=list(model,times)]
+                                                time=time,
+                                                IC0,
+                                                residuals=residuals,
+                                                IC.G=Weights$IC,
+                                                cens.model=cens.model,
+                                                conservative = conservative,
+                                                nth.times=nth.times[1],
+                                                event = status0),by=list(model,times)]
         score.loob <- DT.B[,data.table(Brier=sum(residuals)/N,
                                        se=sd(IF.Brier)/sqrt(N)),by=byvars]
       }
@@ -435,7 +405,7 @@ crossvalPerf.loob.Brier <- function(times,mlevs,se.fit,response.type,NT,Response
 
 crossvalPerf.loob <- function(m,times,mlevs,se.fit,response.type,NT,Response,cens.type,Weights,split.method,N,B,DT.B,data,dolist,alpha,byvars,mlabels,ipa,keep.residuals,conservative,cens.model,response.dim,ID,cause) {
   if (m=="AUC"){
-    return(crossvalPerf.loob.AUC(times,mlevs,se.fit,response.type,NT,Response,cens.type,Weights,split.method,N,B,DT.B,data,dolist,alpha,byvars,mlabels,conservative,cens.model))
+    return(crossvalPerf.loob.AUC(times,mlevs,se.fit,response.type,NT,Response,cens.type,Weights,split.method,N,B,DT.B,data,dolist,alpha,byvars,mlabels,conservative,cens.model,cause))
   }
   if (m=="Brier"){
     return(crossvalPerf.loob.Brier(times,mlevs,se.fit,response.type,NT,Response,cens.type,Weights,split.method,N,B,DT.B,data,dolist,alpha,byvars,mlabels,ipa,keep.residuals,conservative,cens.model,response.dim,ID,cause))
@@ -490,4 +460,4 @@ crossvalPerf.bootcv <- function(m,crossval,se.fit,multi.split.test,keep.cv,byvar
   if (keep.cv)
     out <- c(out,list(cv.score=cv.score,cv.contrasts=cv.contrasts))
   out
-}
+}
\ No newline at end of file
