@@ -3,9 +3,9 @@
 ## Author: Thomas Alexander Gerds
 ## Created: Mar 13 2017 (16:53) 
 ## Version: 
-## Last-Updated: Feb  1 2023 (09:59) 
+## Last-Updated: Dec  9 2023 (09:28) 
 ##           By: Thomas Alexander Gerds
-##     Update #: 233
+##     Update #: 240
 #----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -112,6 +112,10 @@ plotRisk <- function(x,
     if (missing(models)){
         models <- pframe[,levels(model)[1:2]]
     }else{
+        if (any(!(models %in% unique(pframe$models)))){
+            stop(paste0("Fitted object does not contain models named: ",paste0(models,collapse = ", "),
+                        "\nAvailable models are named: ",paste0(unique(pframe$models),collapse = ", ")))
+        }
         if (is.na(models[2])) stop("Need two models for a scatterplot of predicted risks")
         fitted.models <- x$models
         if (is.numeric(models)) {
@@ -282,6 +286,5 @@ plotRisk <- function(x,
     do.call("legend",control$legend)
 }
 
-
 ######################################################################
 ### plotRisk.R ends here
