@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: sep  1 2020 (14:58) 
 ## Version: 
-## Last-Updated: okt  8 2021 (17:19) 
+## Last-Updated: maj  2 2023 (09:44) 
 ##           By: Brice Ozenne
-##     Update #: 332
+##     Update #: 381
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -16,9 +16,9 @@
 ### Code:
 
 ## * wglm
-#' @title Logistic Regression Using IPCW (EXPERIMENTAL!!!)
+#' @title Logistic Regression Using IPCW
 #' @description Logistic regression over multiple timepoints
-#' where right-censoring is handle using inverse probability of censoring weighting. EXPERIMENTAL!!!
+#' where right-censoring is handled using inverse probability of censoring weighting (IPCW). 
 #'
 #' @param regressor.event [formula] a formula with empty left hand side and the covariates for the logistic regression on the right hand side.
 #' @param formula.censor [formula] a formula used to fit the censoring model.
@@ -46,70 +46,31 @@
 #' dSurv <- d[event!=2] ## remove competing risk
 #'
 #' #### no censoring ####
-#' ## wglm
-#' e0.wglm <- wglm(regressor.event = ~ X1, formula.censor = Surv(time,event==0) ~ X1,
-#'                times = tau, data = dFull)
+#' e.wglm <- wglm(regressor.event = ~ X1, formula.censor = Surv(time,event==0) ~ 1,
+#'                times = tau, data = dFull, product.limit = TRUE)
+#' e.wglm ## same as a logistic regression
 #'
-#' ## binreg
-#' if(require(mets)){
-#' e0.binreg <- binreg(formula = Event(time,event)~X1,
-#'                     time = tau[5], data = dFull)
-#' summary(e0.binreg)$coef
-#' summary(e0.wglm$fit[[5]])$coef
-#' ## same
-#' }
-#' 
-#' #### independent censoring (no competing risks) ####
-#' ## wglm 
-#' e.wglm <- wglm(regressor.event = ~ X1-1, formula.censor = Surv(time,event==0) ~ 1,
+#' summary(ate(e.wglm, data = dFull, times = tau, treatment = "X1", verbose = FALSE))
+#'
+#' #### right-censoring ####
+#' ## no covariante in the censoring model (independent censoring)
+#' eC.wglm <- wglm(regressor.event = ~ X1, formula.censor = Surv(time,event==0) ~ 1,
 #'                times = tau, data = dSurv, product.limit = TRUE)
+#' eC.wglm
 #'
-#' ## check weights
-#' e.KM <- predictCoxPL(coxph(Surv(time,event==0) ~ 1, data = dSurv),
-#'                      type = c("hazard","survival"))
-#' if(require(data.table)){
-#' dt.KM <- data.table::as.data.table(e.KM)
-#' dt.KM[times<=tau[1] & hazard!=0, .(times, survival, weight=1/survival)]
-#' }
-#' table(e.wglm$data$XX_IPCW.1_XX)
-#'
-#' ## binreg
-#' if(require(mets)){
-#' e2.binreg <- binreg(formula = Event(time,event)~X1-1,
-#'                    time = tau[1], data = dSurv, cens.code = 0, cause = 1)
-#' table(e2.binreg$cens.weights) ## fixed at prediction time
-#' }
+#' ## with covariates in the censoring model
+#' eC2.wglm <- wglm(regressor.event = ~ X1 + X8,
+#'                 formula.censor = Surv(time,event==0) ~ X1*X8,
+#'                  times = tau, data = dSurv)
+#' eC2.wglm
 #' 
-#' ## "by hand"
-#' if(FALSE){
-#' X <-  model.matrix(~X1-1,dSurv)
-#' Y <-  dSurv$event
-#' w <-  e2.binreg$cens.weights[,1]
-#' p <- predict(e2.binreg, newdata = data.frame(X1=unique(dSurv$X1)))
-#' ## solve the score equation
-#' sum(X[,1] * (w * Y - p[1,1]))
-#' sum(X[,2] * (w * Y - p[2,1]))
-#' }
+#' #### Competing risks ####
+#' ## here Kaplan-Meier as censoring model
+#' eCR.wglm <- wglm(regressor.event = ~ X1,
+#'                  formula.censor = Surv(time,event==0) ~ strata(X1),
+#'                  times = tau, data = d, cause = 1, product.limit = TRUE)
+#' eCR.wglm
 #' 
-#' #### informative censoring (no competing risks) ####
-#' e.wglm <- wglm(regressor.event = ~ X1, formula.censor = Surv(time,event==0) ~ X1,
-#'                times = tau, data = dSurv)
-#'
-#' ## by hand based on the weights
-#' if(FALSE){
-#' X <-  data.frame(X1.0 = e.wglm$data$X1=="0", X1.1 = e.wglm$data$X1=="1")
-#' Y <-  as.data.frame(e.wglm$data)[,grep("event\\.", names(e.wglm$data))]
-#' w <-  as.data.frame(e.wglm$data)[,grep("IPCW\\.", names(e.wglm$data))]
-#' 
-#' p.wglm <- rbind("0" = colSums( (w*Y)[X$X1.0,]) / colSums(w[X$X1.0,]),
-#'                 "1" = colSums( (w*Y)[X$X1.1,]) / colSums(w[X$X1.1,]))
-#' log(p.wglm/(1-p.wglm)) - apply(coef(e.wglm),1,cumsum) ## same as wglm
-#'
-#' ## solve the score equation
-#' sum(w[,1] * X[,1] * (Y[,1] - p.wglm[1,1]))
-#' sum(w[,1] * X[,2] * (Y[,1] - p.wglm[2,1]))
-#' }
-#'
 #' @export
 wglm <- function(regressor.event, formula.censor, times, data, cause = NA,
                  fitter = "coxph", product.limit = FALSE){
@@ -157,22 +118,19 @@ wglm <- function(regressor.event, formula.censor, times, data, cause = NA,
     for(iTime in 1:n.times){
         iFormula.glm <- update(regressor.event, paste0(newname[iTime],"~."))
         data[[newname[iTime]]] <- (data[[varSurv$status]]==cause)*(recoverTime<=times[iTime])
-        data[[obs.newname[iTime]]] <- (1-(recoverCensor==1)*(recoverTime<=times[iTime])) ## equivalent to status>0 or eventtime>tau
+        data[[obs.newname[iTime]]] <- (1-(recoverCensor==1)*(recoverTime<times[iTime])) ## equivalent to status>0 or eventtime>=tau
 
         if(n.censor[iTime]>0){
             iPred <- predictRisk(object.censor, diag = TRUE, newdata = data, times = pmin(recoverTime, times[iTime]) - tol,
                                  type = "survival", product.limit = product.limit)
             data[[IPCW.newname[[iTime]]]] <- ifelse(data[[obs.newname[iTime]]],1/iPred[,1],0)
-            out$fit[[iTime]] <- do.call(stats::glm, list(formula = iFormula.glm, family = stats::quasibinomial(link = "logit"),
-                                                                          data = data, weights = data[[IPCW.newname[[iTime]]]]))
+            out$fit[[iTime]] <- suppressWarnings(do.call(stats::glm, list(formula = iFormula.glm, family = stats::binomial(link = "logit"),
+                                                                          data = data, weights = data[[IPCW.newname[[iTime]]]])))
             ## Warning message:
             ##             In eval(family$initialize) : non-integer #successes in a binomial glm!
 
-            ## alternative not suitable when adding the influence function
-            ## iData <- as.data.frame(data)[data[[obs.newname[iTime]]]==1,,drop=FALSE]
-            ## suppressWarnings(out$fit[[iTime]] <- do.call(stats::glm, list(formula = iFormula.glm, family = stats::binomial(link = "logit"),
-            ##                                                               data = iData, weights = iData[[IPCW.newname[[iTime]]]])))
             out$fit[[iTime]]$time.prior.weights <- pmin(recoverTime, times[iTime]) - tol
+            out$fit[[iTime]]$prior.weights2 <- ifelse(data[[obs.newname[iTime]]],1/iPred[,1]^2,0)
         }else{
             IPCW.newname[[iTime]] <- NA
             out$fit[[iTime]] <- do.call(stats::glm, list(formula = iFormula.glm,
@@ -336,9 +294,13 @@ print.wglm <- function(x, times = NULL, ...){
 #' @param times [numeric vector] time points at which the score should be output. 
 #' @param simplifies [logical] should the ouput be converted to a matrix when only one timepoint is requested. Otherwise will always return a list.
 #' @param ... Not used.
-#'
+#' @export
 score.wglm <- function(x, indiv = FALSE, times = NULL, simplifies = TRUE, ...){
-    if(is.null(times)){
+    if(inherits(x,"glm") && !inherits(x,"wglm")){
+        x <- list(fit = list("1" = x),
+                  times = "1")
+        times <- "1"
+    }else if(is.null(times)){
         times <- x$times
     }else{
         if(any(times %in% x$times == FALSE)){
@@ -378,9 +340,13 @@ score.wglm <- function(x, indiv = FALSE, times = NULL, simplifies = TRUE, ...){
 #' @param times [numeric vector] time points at which the score should be output. 
 #' @param simplifies [logical] should the ouput be converted to a matrix when only one timepoint is requested. Otherwise will always return a list.
 #' @param ... Not used.
-#' 
+#' @export
 information.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
-    if(is.null(times)){
+    if(inherits(x,"glm") && !inherits(x,"wglm")){
+        x <- list(fit = list("1" = x),
+                  times = "1")
+        times <- "1"
+    }else if(is.null(times)){
         times <- x$times
     }else{
         if(any(times %in% x$times == FALSE)){
@@ -418,11 +384,16 @@ information.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
 #' @param x a wglm object.
 #' @param times [numeric vector] time points at which the iid should be output. 
 #' @param simplifies [logical] should the ouput be converted to a matrix when only one timepoint is requested. Otherwise will always return a list.
-
 #' @param ... Not used.
-#' 
+#' @export
 iid.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
-    if(is.null(times)){
+    if(inherits(x,"glm") && !inherits(x,"wglm")){
+        x <- list(fit = list("1" = x),
+                  times = "1",
+                  n.censor = 0)
+        times <- "1"
+        class(x) <- append("wglm",class(x))
+    }else if(is.null(times)){
         times <- x$times
     }else{
         if(any(times %in% x$times == FALSE)){
@@ -432,7 +403,6 @@ iid.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
         }
     }
     n.times <- length(times)
-
     ls.score <- lava::score(x, times = times, simplifies = FALSE, indiv = TRUE)
     ls.info <- lava::information(x, times = times, simplifies = FALSE)
 
@@ -462,12 +432,13 @@ iid.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
         ## NOTE: IF_{W_O}(O_i) = dW(O,\eta)/d\eta \IF_\eta(O_i)
         if(x$n.censor[iTime]>0){
             n.obs <- stats::nobs(iObject)
+            W2 <- iObject$prior.weights2
             X <- stats::model.matrix(iObject)
             pi <- stats::predict(iObject, type = "response")
             Y <- iObject$y
 
             factor <- TRUE        
-            attr(factor,"factor") <- lapply(apply(colMultiply_cpp(X, (Y - pi)), 2, list), function(iVec){cbind(iVec[[1]])})
+            attr(factor,"factor") <- lapply(apply(colMultiply_cpp(X, -W2*(Y - pi)), 2, list), function(iVec){cbind(iVec[[1]])})
             iPred <- predictRisk(x$cox, diag = TRUE, newdata = x$data, times = iObject$time.prior.weights,
                                  type = "survival", product.limit = FALSE, average.iid = factor, store.iid = "minimal")
         }
